@@ -263,7 +263,7 @@ hebrew_engine_shape (PangoEngineShape *engine,
   desc.n_other_features = 0;
   desc.other_features = NULL;
 
-  ruleset = pango_ot_ruleset_get_for (pango_ot_info_get (face), &desc);
+  ruleset = pango_ot_ruleset_get_for_description (pango_ot_info_get (face), &desc);
 
   pango_ot_ruleset_get_feature_count (ruleset, NULL, &n_gpos_features);
   if (n_gpos_features == 0)
