@@ -162,7 +162,7 @@ syriac_engine_shape (PangoEngineShape *engine,
   desc.n_other_features = 0;
   desc.other_features = NULL;
 
-  ruleset = pango_ot_ruleset_get_for (pango_ot_info_get (face), &desc);
+  ruleset = pango_ot_ruleset_get_for_description (pango_ot_info_get (face), &desc);
 
   pango_ot_ruleset_substitute (ruleset, buffer);
   pango_ot_ruleset_position (ruleset, buffer);
